@@ -104,6 +104,16 @@ cairo_test_init (const char *test_name)
     printf ("\nTESTING %s\n", test_name);
 }
 
+void
+cairo_test_fini (void)
+{
+    fclose (cairo_test_log_file);
+    cairo_debug_reset_static_data ();
+#if HAVE_FCFINI
+    FcFini ();
+#endif
+}
+
 void
 cairo_test_log (const char *fmt, ...)
 {
@@ -287,8 +297,6 @@ cairo_test_for_target (cairo_test_t			 *test,
 	goto UNWIND_CAIRO;
     }
 
-    cairo_show_page (cr);
-
     if (cairo_status (cr) != CAIRO_STATUS_SUCCESS) {
 	cairo_test_log ("Error: Function under test left cairo status in an error state: %s\n",
 			cairo_status_to_string (cairo_status (cr)));
@@ -554,14 +562,10 @@ cairo_test_expecting (cairo_test_t *test,
 	ret = CAIRO_TEST_FAILURE;
     }
 
-    fclose (cairo_test_log_file);
+    cairo_test_fini ();
 
     free (targets_to_test);
 
-#if HAVE_FCFINI
-    FcFini ();
-#endif
-
     return ret;
 }
 
